'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _Source2 = require('./Source');

var _Source3 = _interopRequireDefault(_Source2);

var _URLBuilder = require('../Provider/URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WFSSource = function (_Source) {
    (0, _inherits3.default)(WFSSource, _Source);

    /**
     * Features source
     * @constructor
     * @extends Source
     *
     * @param {sourceParams}  source
     * @param {string} source.typeName Name of the feature type to describe
     * @param {string} source.projection crs of wfs
     * @param {string} [source.version='2.0.2'] wfs protocol version
     * @param {Object} [source.zoom]
     * @param {number} [source.zoom.min] layer's zoom minimum
     * @param {number} [source.zoom.max] layer's zoom maximum
     *
     * @example <caption>Add color layer with wfs source</caption>
     * const colorlayer = new ColorLayer('color_build', {
     *     style: {
     *         fill: 'red',
     *         fillOpacity: 0.5,
     *         stroke: 'white',
     *     },
     *     source: {
     *        url: 'http://wxs.fr/wfs',
     *        protocol: 'wfs',
     *        version: '2.0.0',
     *        typeName: 'BDTOPO_BDD_WLD_WGS84G:bati_remarquable',
     *        projection: 'EPSG:4326',
     *        extent: {
     *            west: 4.568,
     *            east: 5.18,
     *            south: 45.437,
     *            north: 46.03,
     *        },
     *        format: 'application/json',
     *     }
     * });
     * // Add the layer
     * view.addLayer(colorlayer);
     *
     * @example <caption>Add geometry layer with wfs source</caption>
     * const geometrylayer = new GeometryLayer('mesh_build', {
     *     update: itowns.FeatureProcessing.update,
     *     convert: itowns.Feature2Mesh.convert({ extrude: () => 50 }),
     *     source: {
     *        protocol: 'wfs',
     *        url: 'http://wxs.fr/wfs',
     *        version: '2.0.0',
     *        typeName: 'BDTOPO_BDD_WLD_WGS84G:bati_remarquable',
     *        projection: 'EPSG:4326',
     *        extent: {
     *            west: 4.568,
     *            east: 5.18,
     *            south: 45.437,
     *            north: 46.03,
     *        },
     *        zoom: { min: 14, max: 14 },
     *        format: 'application/json',
     *     }
     * });
     * // Add the layer
     * view.addLayer(geometrylayer);
    */
    function WFSSource(source) {
        (0, _classCallCheck3.default)(this, WFSSource);

        if (!source.typeName) {
            throw new Error('source.typeName is required in wfs source.');
        }

        if (!source.projection) {
            throw new Error('source.projection is required in wfs source');
        }

        var _this = (0, _possibleConstructorReturn3.default)(this, (WFSSource.__proto__ || (0, _getPrototypeOf2.default)(WFSSource)).call(this, source));

        _this.typeName = source.typeName;
        _this.format = _this.format || 'application/json';
        _this.version = source.version || '2.0.2';

        _this.url = source.url + 'SERVICE=WFS&REQUEST=GetFeature&typeName=' + _this.typeName + '&VERSION=' + _this.version + '&SRSNAME=' + _this.projection + '&outputFormat=' + _this.format + '&BBOX=%bbox,' + _this.projection;

        _this.zoom = source.zoom || { min: 0, max: 21 };
        return _this;
    }

    (0, _createClass3.default)(WFSSource, [{
        key: 'handlingError',
        value: function handlingError(err, url) {
            var _this2 = this;

            if (err.response && err.response.status == 400) {
                return err.response.text().then(function (text) {
                    var getCapUrl = _this2.url + 'SERVICE=WFS&REQUEST=GetCapabilities&VERSION=' + _this2.version;
                    var xml = new DOMParser().parseFromString(text, 'application/xml');
                    var errorElem = xml.querySelector('Exception');
                    var errorCode = errorElem.getAttribute('exceptionCode');
                    var errorMessage = errorElem.querySelector('ExceptionText').textContent;
                    console.error('Source ' + _this2.typeName + ': bad request when fetching data. Server says: "' + errorCode + ': ' + errorMessage + '". \nReviewing ' + getCapUrl + ' may help.', err);
                    throw err;
                });
            } else {
                console.error('Source ' + this.typeName + ': error while trying to fetch/parse/convert WFS data. Url was ' + url + '.', err);
                throw err;
            }
        }
    }, {
        key: 'urlFromExtent',
        value: function urlFromExtent(extent) {
            return _URLBuilder2.default.bbox(extent.as(this.projection), this);
        }
    }, {
        key: 'extentInsideLimit',
        value: function extentInsideLimit(extent) {
            return (extent.zoom == undefined || extent.zoom >= this.zoom.min && extent.zoom <= this.zoom.max) && this.extent.intersectsExtent(extent);
        }
    }]);
    return WFSSource;
}(_Source3.default);

exports.default = WFSSource;